<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) exit;

class Gooyapay extends PaymentModule
{

	private $_html = '';
	private $_postErrors = array();

	public function __construct()
	{
		$this->name = 'gooyapay';
		$this->tab = 'payments_gateways';
		$this->version = '2.5';
		$this->author = 'https://modulebank.ir';
		$this->currencies = true;
		$this->currencies_mode = 'checkbox';
		$this->controllers = array('request', 'verify');
		parent::__construct();
		$this->page = basename(__FILE__, '.php');
		$this->displayName = 'درگاه پرداخت گویاپـــی';
		$this->description = 'پرداخت از طریق درگاه پرداخت گویاپـــی';
		$this->confirmUninstall = 'آیا مطمئن هستید می خواهید جزئیات را حذف کنید ؟';
		$this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
		if (!sizeof(Currency::checkPaymentCurrencies($this->id)))
			$this->warning = 'لطفا واحد پولی درگاه را تنظیم نمایید';
		$config = Configuration::getMultiple(array('gooyapay_merchant', 'gooyapay_title', 'gooyapay_rate', ''));
		if (!isset($config['gooyapay_merchant'],$config['gooyapay_title'],$config['gooyapay_rate']))
			$this->warning = 'لطفا اطلاعات پذیرنده درگاه را تنظیم نمایید';
	}

	public function install()
	{
		if (!parent::install() or !Configuration::updateValue('gooyapay_merchant', '') or !Configuration::updateValue('gooyapay_title', '') or !Configuration::updateValue('gooyapay_rate', '') or !$this->registerHook('paymentOptions') or !$this->registerHook('paymentReturn'))
		return false;
		return true;
	}

	public function uninstall()
	{
		if (!Configuration::deleteByName('gooyapay_merchant') or !Configuration::deleteByName('gooyapay_title') or !Configuration::deleteByName('gooyapay_rate') or !parent::uninstall())
		return false;
		return true;
	}

	public function displayFormSettings()
	{
		$this->_html .= '<form action="'.$_SERVER['REQUEST_URI'].'" method="post">
			<fieldset>
				<legend>تنظیمات</legend>
				<label> عنوان نمایشی درگاه گویاپـــی </label>
				<div class="margin-form"><input type="text" size="30" name="gooyapay_title" id="gooyapay_title" value="'.Configuration::get('gooyapay_title').'" /></div>
				<label> شماره پذیرنده درگاه گویاپـــی </label>
				<div class="margin-form"><input type="text" size="30" name="gooyapay_merchant" id="gooyapay_merchant" value="'.Configuration::get('gooyapay_merchant').'" /></div>
				<label> نرخ ارز </label>
				<div class="margin-form"><input type="number" min="0.01" step="0.01" size="30" name="gooyapay_rate" id="gooyapay_rate" value="'.(Configuration::get('gooyapay_rate')?:1).'" /></div>
				<center><input type="submit" name="submit" value="ذخیره" class="button" /></center>
			</fieldset>
		</form>';
	}

	public function displayConf()
	{
		$this->_html .= '<div class="conf confirm">تنظیمات بروز رسانی شد</div>';
	}

	public function displayErrors()
	{
		foreach ($this->_postErrors as $err)
		$this->_html .= '<div class="alert error">'. $err .'</div>';
	}

	public function getContent()
	{
		$this->displayGooyapay();
		$this->postprocess();
		$this->displayFormSettings();
		return $this->_html;
	}
	
	private function postprocess()
	{
		if (isset($_POST['submit']))
		{
			if (empty($_POST['gooyapay_merchant']))
				$this->_postErrors[] = 'ورود شماره پذیرنده درگاه گویاپـــی الزامیست.';
			if (empty($_POST['gooyapay_title']))
				$this->_postErrors[] = 'ورود نام نمایشی درگاه گویاپـــی الزامیست.';
			if (empty($_POST['gooyapay_rate']))
				$this->_postErrors[] = 'ورود نرخ ارز درگاه گویاپـــی الزامیست.';
			if (!sizeof($this->_postErrors))
			{
				Configuration::updateValue('gooyapay_merchant', $_POST['gooyapay_merchant']);
				Configuration::updateValue('gooyapay_title', $_POST['gooyapay_title']);
				Configuration::updateValue('gooyapay_rate', $_POST['gooyapay_rate']);
				return $this->displayConf();
			}
			return $this->displayErrors();
		}
	}

	private function displayGooyapay()
	{
		$this->_html .= '<h2>درگاه پرداخت گویاپـــی</h2><img src="../modules/gooyapay/logo.png" style="float:left; margin-right:15px;"><b>این افزونه امکان پرداخت از طریق درگاه پرداخت گویاپـــی را برای شما فراهم می کند.</b><br /><br />';
	}

	public function execPayment($cart)
	{
		global $cookie, $smarty;
		$amount = $cart->getOrderTotal(true, Cart::BOTH);
		$rate = Configuration::get('gooyapay_rate')?:1;
		if ($rate > 0) $amount *= $rate;
		$amount = intval($amount);
		$ch = curl_init('https://gooyapay.ir/webservice/rest/PaymentRequest');
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
		curl_setopt($ch, CURLOPT_POSTFIELDS, $b = json_encode($p = [
				'MerchantID' 	=> Configuration::get('gooyapay_merchant'),
				'Amount' 		=> $amount,
				'InvoiceID' 	=> $cart->id,
				'Description' 	=> 'PS17_Order_' . $cart->id,
				'CallbackURL' 	=> $this->context->link->getModuleLink($this->name, 'verify', array(), true),
				'RequestMethod' => 'get'
			]));
		curl_setopt($ch, CURLOPT_HTTPHEADER, $h = array(
				'Content-Type: application/json',
				'Accept: application/json'
			));
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER , true);
		$response = curl_exec($ch);
		$error = curl_error($ch);
		$info = curl_getinfo($ch);
		curl_close($ch);
		$result = json_decode($response);
		if (isset($result->Status) && $result->Status == 100) {
			@header('location: ' . $result->PaymentUrl);
			Tools::redirectLink($result->PaymentUrl);
			return '<p>در حال اتصال به بانک ...</p><script>document.location="'.$result->PaymentUrl.'";</script>';
		} elseif (isset($result->Message) && $result->Message != '') {
			return 'Error : ' . $result->Message;
		} elseif ($error) {
			return 'Connecting Error : ' . $error;
		} else {
			return 'UnExpected Error in connecting to web service';
		}
	}

	public function confirmPayment($amount)
	{
		if (isset($_GET['PaymentStatus']) && $_GET['PaymentStatus'] == 'OK') {
			$rate = Configuration::get('gooyapay_rate')?:1;
			if ($rate > 0) $amount *= $rate;
			$amount = intval($amount);
			$ch = curl_init('https://gooyapay.ir/webservice/rest/PaymentVerification');
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
			curl_setopt($ch, CURLOPT_POSTFIELDS, $b = json_encode($p = [
					'MerchantID' 	=> Configuration::get('gooyapay_merchant'),
					'Amount' 		=> $amount,
					'Authority' 	=> $_GET['Authority']
				]));
			curl_setopt($ch, CURLOPT_HTTPHEADER, $h = array(
					'Content-Type: application/json',
					'Accept: application/json'
				));
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER , true);
			$response = curl_exec($ch);
			$error = curl_error($ch);
			$info = curl_getinfo($ch);
			curl_close($ch);
			$result = json_decode($response);
			if (isset($result->Status) && $result->Status == 100) {
				return array(true, $result->RefID);
			} elseif (isset($result->Message) && $result->Message != '') {
				$err = 'Error : ' . $result->Message;
				return array(false, $err);
			} elseif ($error) {
				$err = 'Connecting Error : ' . $error;
				return array(false, $err);
			} else {
				$err = 'UnExpected Error in connecting to web service';
				return array(false, $err);
			}
		} else {
			return array(false, 'تراکنش از سوی کاربر لغو شد');
		}
	}

	public function showMessages($result)
	{
		$this->_postErrors[] = $result;
		$this->displayErrors();
		echo $this->_html;
		return $result;
	}

	public function hookPaymentOptions($params)
	{
		if (!$this->active) return;
		//if (!$this->checkCurrency($params['cart'])) return;
		$cart = $this->context->cart;
		$total = Tools::displayPrice($cart->getOrderTotal(true, Cart::BOTH));
		$view = '<section><p class="payment_module"><img src="modules/gooyapay/logo.png" alt="'.Configuration::get('gooyapay_title').'" />'.Configuration::get('gooyapay_title').' <br /></p></section>';
		$newOption = new PaymentOption();
		$newOption->setModuleName($this->name)->setCallToActionText(Configuration::get('gooyapay_title'))->setAction($this->context->link->getModuleLink($this->name, 'request', array(), true))->setAdditionalInformation($view);
		return array($newOption);
	}
}
